"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const injection_handler_1 = require("./injection-handler");
const container_binding_config_1 = require("./container-binding-config");
/**
 * Internal implementation of IoC Container.
 */
class IoCContainer {
    static bind(source) {
        injection_handler_1.InjectorHandler.checkType(source);
        const baseSource = injection_handler_1.InjectorHandler.getConstructorFromType(source);
        let config = IoCContainer.bindings.get(baseSource);
        if (!config) {
            config = new container_binding_config_1.IoCBindConfig(baseSource, IoCContainer.get, IoCContainer.getValue);
            config
                .to(source);
            IoCContainer.bindings.set(baseSource, config);
        }
        return config;
    }
    static bindName(name) {
        injection_handler_1.InjectorHandler.checkName(name);
        const property = container_binding_config_1.PropertyPath.parse(name);
        let config = IoCContainer.values.get(property.name);
        if (!config) {
            config = new container_binding_config_1.IoCBindValueConfig(property.name);
            IoCContainer.values.set(property.name, config);
        }
        config.path = property.path;
        return config;
    }
    static get(source, context) {
        const config = IoCContainer.bind(source);
        if (!config.iocFactory) {
            config.to(config.source);
        }
        return config.getInstance(context);
    }
    static getValue(name) {
        const config = IoCContainer.bindName(name);
        return config.getValue();
    }
    static getType(source) {
        injection_handler_1.InjectorHandler.checkType(source);
        const baseSource = injection_handler_1.InjectorHandler.getConstructorFromType(source);
        const config = IoCContainer.bindings.get(baseSource);
        if (!config) {
            throw new TypeError(`The type ${source.name} hasn't been registered with the IOC Container`);
        }
        return config.targetSource || config.source;
    }
    static injectProperty(target, key, propertyType) {
        injection_handler_1.InjectorHandler.injectProperty(target, key, propertyType, IoCContainer.get);
    }
    static injectValueProperty(target, key, name) {
        injection_handler_1.InjectorHandler.injectValueProperty(target, key, name, IoCContainer.getValue);
    }
    /**
     * Store the state for a specified binding.  Can then be restored later.   Useful for testing.
     * @param source The dependency type
     */
    static snapshot(source) {
        const config = IoCContainer.bind(source);
        IoCContainer.snapshots.set(source, {
            factory: config.iocFactory,
            scope: config.iocScope,
            withParams: config.paramTypes
        });
        return {
            restore: () => IoCContainer.restore(source)
        };
    }
    /**
     * Restores the state for a specified binding that was previously captured by snapshot.
     * @param source The dependency type
     */
    static restore(source) {
        const config = IoCContainer.bind(source);
        const configSnapshot = IoCContainer.snapshots.get(source);
        config
            .factory(configSnapshot.factory)
            .scope(configSnapshot.scope)
            .withParams(configSnapshot.withParams);
    }
}
exports.IoCContainer = IoCContainer;
/**
 * Internal storage for snapshots
 * @type {providers: Map<Function, Provider>; scopes: Map<Function, Scope>}
 */
IoCContainer.snapshots = new Map();
IoCContainer.bindings = new Map();
IoCContainer.values = new Map();
//# sourceMappingURL=container.js.map